@@ -4,6 +4,7 @@ import datetime
 import functools
 import inspect
 import logging
+import pydantic
 from types import TracebackType
 from typing import (
     Any,
@@ -16,17 +17,25 @@ from typing import (
     Callable,
     Awaitable,
     TYPE_CHECKING,
+    overload,
 )
 
 import httpx
 from typing_extensions import ParamSpec, override, get_origin
-
+from ._types import NoneType
 from ._constants import CLIENT_REQUEST_HEADER, RAW_RESPONSE_HEADER  # type: ignore
 from ._exceptions import ArkError
-from ._streaming import Stream, AsyncStream
-from ._utils import extract_type_arg, is_annotated_type  # type: ignore
+from ._utils import is_given, extract_type_arg, is_annotated_type, is_type_alias_type
+from ._streaming import (
+    Stream,
+    AsyncStream,
+    is_stream_class_type,
+    extract_stream_chunk_type,
+)
+from ._models import BaseModel, is_basemodel, add_request_id
 
 if TYPE_CHECKING:
+    from ._request_options import RequestOptions
     from ._base_client import BaseClient
 
 P = ParamSpec("P")
@@ -46,6 +55,7 @@ class BaseAPIResponse(Generic[R]):
     _is_sse_stream: bool
     _stream_cls: type[Stream[Any]] | type[AsyncStream[Any]] | None
     _request_id: str
+    _options: RequestOptions
 
     http_response: httpx.Response
 
@@ -57,6 +67,7 @@ class BaseAPIResponse(Generic[R]):
         client: BaseClient,
         stream: bool,
         stream_cls: type[Stream[Any]] | type[AsyncStream[Any]] | None,
+        options: RequestOptions,
     ) -> None:
         self._cast_to = cast_to
         self._client = client
@@ -64,6 +75,7 @@ class BaseAPIResponse(Generic[R]):
         self._is_sse_stream = stream
         self._stream_cls = stream_cls
         self.http_response = raw
+        self._options = options
 
     @property
     def request_id(self) -> str:
@@ -105,26 +117,84 @@ class BaseAPIResponse(Generic[R]):
     def __repr__(self) -> str:
         return f"<{self.__class__.__name__} [{self.status_code} {self.http_response.reason_phrase}] type={self._cast_to}>"
 
-    def _parse(self) -> R:
-        if self._is_sse_stream:
-            if self._stream_cls is None:
-                raise MissingStreamClassError()
+    def _parse(self, *, to: type[_T] | None = None) -> R | _T:
+        cast_to = to if to is not None else self._cast_to
 
-            return cast(
-                R,
-                self._stream_cls(
-                    cast_to=extract_type_arg(self._stream_cls),
-                    response=self.http_response,
-                    client=self._client,  # type: ignore
-                ),
-            )
+        # unwrap `TypeAlias('Name', T)` -> `T`
+        if is_type_alias_type(cast_to):
+            cast_to = cast_to.__value__  # type: ignore[unreachable]
 
-        cast_to = self._cast_to
-        if is_annotated_type(cast_to):
+        # unwrap `Annotated[T, ...]` -> `T`
+        if cast_to and is_annotated_type(cast_to):
             cast_to = extract_type_arg(cast_to, 0)
 
         origin = get_origin(cast_to) or cast_to
 
+        if self._is_sse_stream:
+            if to:
+                if not is_stream_class_type(to):
+                    raise TypeError(
+                        f"Expected custom parse type to be a subclass of {Stream} or {AsyncStream}"
+                    )
+
+                return cast(
+                    _T,
+                    to(
+                        cast_to=extract_stream_chunk_type(
+                            to,
+                            failure_message="Expected custom stream type to be passed with a type argument, e.g. Stream[ChunkType]",
+                        ),
+                        response=self.http_response,
+                        client=cast(Any, self._client),
+                    ),
+                )
+
+            if self._stream_cls:
+                return cast(
+                    R,
+                    self._stream_cls(
+                        cast_to=extract_stream_chunk_type(self._stream_cls),
+                        response=self.http_response,
+                        client=cast(Any, self._client),
+                    ),
+                )
+
+            stream_cls = cast(
+                "type[Stream[Any]] | type[AsyncStream[Any]] | None",
+                self._client._default_stream_cls,
+            )
+            if stream_cls is None:
+                raise MissingStreamClassError()
+
+            return cast(
+                R,
+                stream_cls(
+                    cast_to=cast_to,
+                    response=self.http_response,
+                    client=cast(Any, self._client),
+                ),
+            )
+
+        if cast_to is NoneType:
+            return cast(R, None)
+
+        response = self.http_response
+        if cast_to == str:
+            return cast(R, response.text)
+
+        if cast_to == bytes:
+            return cast(R, response.content)
+
+        if cast_to == int:
+            return cast(R, int(response.text))
+
+        if cast_to == float:
+            return cast(R, float(response.text))
+
+        if cast_to == bool:
+            return cast(R, response.text.lower() == "true")
+
+        # handle the legacy binary response case
         if (
             inspect.isclass(cast_to)
             and cast_to.__name__ == "HttpxBinaryResponseContent"
@@ -132,15 +202,47 @@ class BaseAPIResponse(Generic[R]):
             return cast(R, cast_to(response))  # type: ignore
 
         if origin == ArkAPIResponse:
-            raise RuntimeError("Unexpected state - cast_to is `ArkAPIResponse`")
+            raise RuntimeError("Unexpected state - cast_to is `APIResponse`")
 
-        content_type, *_ = self.http_response.headers.get("content-type", "*").split(
-            ";"
-        )
+        if inspect.isclass(origin) and issubclass(origin, httpx.Response):
+            # Because of the invariance of our ResponseT TypeVar, users can subclass httpx.Response
+            # and pass that class to our request functions. We cannot change the variance to be either
+            # covariant or contravariant as that makes our usage of ResponseT illegal. We could construct
+            # the response class ourselves but that is something that should be supported directly in httpx
+            # as it would be easy to incorrectly construct the Response object due to the multitude of arguments.
+            if cast_to != httpx.Response:
+                raise ValueError(
+                    "Subclasses of httpx.Response cannot be passed to `cast_to`"
+                )
+            return cast(R, response)
+
+        if (
+            inspect.isclass(
+                origin  # pyright: ignore[reportUnknownArgumentType]
+            )
+            and not issubclass(origin, BaseModel)
+            and issubclass(origin, pydantic.BaseModel)
+        ):
+            raise TypeError("Pydantic models must subclass our base model type")
+
+        if (
+            cast_to is not object
+            and origin is not list
+            and origin is not dict
+            and origin is not Union
+            and not issubclass(origin, BaseModel)
+        ):
+            raise RuntimeError(
+                f"Unsupported type, expected {cast_to} to be a subclass of {BaseModel}, {dict}, {list}, {Union}, {NoneType}, {str} or {httpx.Response}."
+            )
+
+        # split is required to handle cases where additional information is included
+        # in the response, e.g. application/json; charset=utf-8
+        content_type, *_ = response.headers.get("content-type", "*").split(";")
         if content_type != "application/json":
-            if cast_to is object:
+            if is_basemodel(cast_to):
                 try:
-                    data = self.http_response.json()
+                    data = response.json()
                 except Exception as exc:
                     log.debug(
                         "Could not read JSON from response data due to %s - %s",
@@ -148,16 +250,23 @@ class BaseAPIResponse(Generic[R]):
                         exc,
                     )
                 else:
-                    return cast(R, data)
+                    return self._client._process_response_data(
+                        data=data,
+                        cast_to=cast_to,  # type: ignore
+                        response=response,
+                    )
 
+            # If the API responds with content that isn't JSON then we just return
+            # the (decoded) text without performing any parsing so that you can still
+            # handle the response however you need to.
             return response.text  # type: ignore
 
-        data = self.http_response.json()
+        data = response.json()
 
         return self._client._process_response_data(
             data=data,
             cast_to=cast_to,  # type: ignore
-            response=self.http_response,
+            response=response,
         )
 
 
@@ -166,10 +275,50 @@ class ArkAPIResponse(BaseAPIResponse[R]):
     def request_id(self) -> str:
         return self.http_response.headers.get(CLIENT_REQUEST_HEADER, "")
 
-    def parse(self) -> R:
+    def parse(self, *, to: type[_T] | None = None) -> R | _T:
+        """Returns the rich python representation of this response's data.
+
+        For lower-level control, see `.read()`, `.json()`, `.iter_bytes()`.
+
+        You can customise the type that the response is parsed into through
+        the `to` argument, e.g.
+
+        ```py
+        class MyModel(BaseModel):
+            foo: str
+
+
+        obj = response.parse(to=MyModel)
+        print(obj.foo)
+        ```
+
+        We support parsing:
+          - `BaseModel`
+          - `dict`
+          - `list`
+          - `Union`
+          - `str`
+          - `int`
+          - `float`
+          - `httpx.Response`
+        """
+        cache_key = to if to is not None else self._cast_to
+        cached = self._parsed_by_type.get(cache_key)
+        if cached is not None:
+            return cached  # type: ignore[no-any-return]
+
         if not self._is_sse_stream:
             self.read()
-        return self._parse()
+
+        parsed = self._parse(to=to)
+        if is_given(self._options.post_parser):
+            parsed = self._options.post_parser(parsed)
+
+        if isinstance(parsed, BaseModel):
+            add_request_id(parsed, self.request_id)
+
+        self._parsed_by_type[cache_key] = parsed
+        return cast(R, parsed)
 
     def read(self) -> bytes:
         try:
@@ -206,34 +355,96 @@ class ArkAsyncAPIResponse(BaseAPIResponse[R]):
     def request_id(self) -> str:
         return self.http_response.headers.get(CLIENT_REQUEST_HEADER, "")
 
-    async def parse(self) -> R:
+    @overload
+    async def parse(self, *, to: type[_T]) -> _T: ...
+
+    @overload
+    async def parse(self) -> R: ...
+
+    async def parse(self, *, to: type[_T] | None = None) -> R | _T:
+        """Returns the rich python representation of this response's data.
+
+        For lower-level control, see `.read()`, `.json()`, `.iter_bytes()`.
+
+        You can customise the type that the response is parsed into through
+        the `to` argument, e.g.
+
+        ```py
+        class MyModel(BaseModel):
+            foo: str
+
+
+        obj = response.parse(to=MyModel)
+        print(obj.foo)
+        ```
+
+        We support parsing:
+          - `BaseModel`
+          - `dict`
+          - `list`
+          - `Union`
+          - `str`
+          - `httpx.Response`
+        """
+        cache_key = to if to is not None else self._cast_to
+        cached = self._parsed_by_type.get(cache_key)
+        if cached is not None:
+            return cached  # type: ignore[no-any-return]
+
         if not self._is_sse_stream:
             await self.read()
 
-        return self._parse()
+        parsed = self._parse(to=to)
+        if is_given(self._options.post_parser):
+            parsed = self._options.post_parser(parsed)
+
+        if isinstance(parsed, BaseModel):
+            add_request_id(parsed, self.request_id)
+
+        self._parsed_by_type[cache_key] = parsed
+        return cast(R, parsed)
 
     async def read(self) -> bytes:
+        """Read and return the binary response content."""
         try:
             return await self.http_response.aread()
         except httpx.StreamConsumed as exc:
+            # the default error raised by httpx isn't very
+            # helpful in our case so we re-raise it with
+            # a different error message
             raise StreamAlreadyConsumed() from exc
 
     async def text(self) -> str:
+        """Read and decode the response content into a string."""
         await self.read()
         return self.http_response.text
 
     async def json(self) -> object:
+        """Read and decode the JSON response content."""
         await self.read()
         return self.http_response.json()
 
     async def close(self) -> None:
+        """Close the response and release the connection.
+
+        Automatically called if the response body is read to completion.
+        """
         await self.http_response.aclose()
 
     async def iter_bytes(self, chunk_size: int | None = None) -> AsyncIterator[bytes]:
+        """
+        A byte-iterator over the decoded response content.
+
+        This automatically handles gzip, deflate and brotli encoded responses.
+        """
         async for chunk in self.http_response.aiter_bytes(chunk_size):
             yield chunk
 
     async def iter_text(self, chunk_size: int | None = None) -> AsyncIterator[str]:
+        """A str-iterator over the decoded response content
+        that handles both gzip, deflate, etc but also detects the content's
+        string encoding.
+        """
         async for chunk in self.http_response.aiter_text(chunk_size):
             yield chunk
 
