@@ -188,6 +188,8 @@ class Completions(SyncAPIResource):
         service_tier: Optional[Literal["auto", "default"]] | None = None,
         tool_choice: ChatCompletionToolChoiceOptionParam | None = None,
         response_format: completion_create_params.ResponseFormat | None = None,
+        thinking: completion_create_params.Thinking | None = None,
+        max_completion_tokens: Optional[int] | None = None,
         user: str | None = None,
         extra_headers: Headers | None = None,
         extra_query: Query | None = None,
@@ -232,6 +234,8 @@ class Completions(SyncAPIResource):
                         "service_tier": service_tier,
                         "tool_choice": tool_choice,
                         "response_format": response_format,
+                        "thinking": thinking,
+                        "max_completion_tokens": max_completion_tokens,
                     },
                     options=make_request_options(
                         extra_headers=extra_headers,
@@ -336,6 +340,8 @@ class AsyncCompletions(AsyncAPIResource):
         service_tier: Optional[Literal["auto", "default"]] | None = None,
         tool_choice: ChatCompletionToolChoiceOptionParam | None = None,
         response_format: completion_create_params.ResponseFormat | None = None,
+        thinking: completion_create_params.Thinking | None = None,
+        max_completion_tokens: Optional[int] | None = None,
         extra_headers: Headers | None = None,
         extra_query: Query | None = None,
         extra_body: Body | None = None,
@@ -380,6 +386,8 @@ class AsyncCompletions(AsyncAPIResource):
                         "service_tier": service_tier,
                         "tool_choice": tool_choice,
                         "response_format": response_format,
+                        "thinking": thinking,
+                        "max_completion_tokens": max_completion_tokens,
                     },
                     options=make_request_options(
                         extra_headers=extra_headers,
